package com.gic.auth.web.controller;

import com.gic.auth.constant.UserConstants;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.qo.UserListQO;
import com.gic.auth.service.UserApiService;
import com.gic.auth.web.utils.UserPasswordUtils;
import com.gic.auth.web.vo.UserDetailVO;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;

@RestController
@RequestMapping("/user")
public class UserController {

    private final Logger logger = LogManager.getLogger(UserController.class);

    @Autowired
    private UserApiService userApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;

    @RequestMapping("/save-or-update-user")
    public RestResponse editUser(@Validated({UserDTO.UserQoValid.class}) UserDTO userDTO) {
        if (UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin().intValue() == 0) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "子管理员不能编辑");
        }
        Integer operationPassword = userDTO.getOperPasswordType();
        userDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());

        boolean isNeedSendSms = false;
        //发送短信的参数
        String[] smsArr = new String[0];
        if (operationPassword != null && operationPassword.intValue() == 1) {
            //如果是1，则不用验证密码

        } else {
            String password = UserPasswordUtils.validPassword(userDTO.getPasswordType(), userDTO.getPassword());
            userDTO.setPassword(UserPasswordUtils.getEncryptPassword(password));
            isNeedSendSms = UserConstants.CREATE_AUTO == userDTO.getPasswordType().intValue();
            if (isNeedSendSms) {
                //发送短信，通知自动生成密码
                if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
                    userDTO.setPhoneAreaCode("86");
                }
                smsArr = new String[]{UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName(), userDTO.getPhoneNumber(), password};
            }
        }
        ServiceResponse userResult;
        //普通用户
        userDTO.setSuperAdmin(0);
        if (userDTO.getUserId() == null) {
            userResult = userApiService.saveUser(userDTO);
        } else {
            userResult = userApiService.editUser(userDTO);
        }
        if (userResult.isSuccess()) {
            if (isNeedSendSms) {
                ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms("GICpassword001",
                        UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseId(), userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
                if (!smsSendResult.isSuccess()) {
                    logger.warn(smsSendResult.getMessage());
                }
            }
            if (userDTO.getUserId() == null) {
                LogUtils.createLog(OperationResultUtils.LOG_NEW + "管理员", userDTO.getUserName());
            } else {
                LogUtils.createLog(OperationResultUtils.LOG_EDIT + "管理员", userDTO.getUserName());
            }
            return RestResponse.success();
        } else {
            return EnterpriseRestResponse.failure(userResult);
        }
    }

    /**
     * 用户详情
     *
     * @param userId
     * @return com.gic.commons.webapi.reponse.RestResponse
     *
     * @Title: getUserById
     * @Description:   * @author guojuxing
     */
    @RequestMapping("/get-user-by-id")
    public RestResponse getUserById(Integer userId) {
        return ResultControllerUtils.commonResultOne(userApiService.getUserById(userId), UserDetailVO.class);
    }

    @RequestMapping("/delete-user-by-id")
    public RestResponse delete(Integer userId) {
        ServiceResponse<String> result = userApiService.delete(userId);
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_DELETE + "管理员",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/list-user")
    public RestResponse listUser(UserListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(userApiService.pageUser(params));
    }

    @RequestMapping("/list-user-nopage")
    public RestResponse listUserNoPage(UserListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(userApiService.listUser(params));
    }

    /**
     * 逻辑删除用户对应分组数据
     *
     * @param type
     * @param accountGroupId
     * @param userId
     * @return com.gic.commons.webapi.reponse.RestResponse
     *
     * @Title: removeUserAccountGroup
     * @Description:   * @author guojuxing
     */
    @RequestMapping("/remove-user-account-group")
    public RestResponse removeUserAccountGroup(Integer type, Integer accountGroupId, Integer userId) {
        ServiceResponse<String> result = userApiService.removeAccountGroup(accountGroupId, userId, type);
        return OperationResultUtils.operationResult(result, "移除账号分组",
                OperationResultUtils.getOperationObject(result));
    }

}
